# Integrate the constant function with value 1 over the circle of radius 1
# centred at the origin.  The value of the integral should be pi.

# Inner integral, over y from -sqrt(1-x^2) to +sqrt(1-x^2).  Note that
# the x passed may be a vector of values, not just a single number, and
# we need to return a vector of function values (here the constant 1) 
# at all these x values.

i1a <- function (x) {
  r <- numeric(length(x))
  for (i in 1:length(x))
    r[i] <- integrate (function(y) rep(1,length(y)),
                       -sqrt(1-x[i]^2), +sqrt(1-x[i]^2)) $ value
  r
}

# Outer integral, over x from -1 to 1.

i2a <- function () 
  integrate(i1a,-1,1) $ value


# Integrate the function with value 1/sqrt(x^2+y^2) over the circle of 
# radius 1 centred at the origin.  The value of the integral should be 2*pi.

# Inner integral, over y from -sqrt(1-x^2) to +sqrt(1-x^2).  Symmetry is
# exploited here, to change the integral from -1 to 1 into two times the
# integral from 0 to 1, and the zero lower bound is slightly fudged to avoid
# the problem that when x is zero, the integral all the way to zero diverges.

i1b <- function (x) {
  r <- numeric(length(x))
  for (i in 1:length(x))
    r[i] <- 2 * integrate (function(y) 1/sqrt(x[i]^2+y^2),
                           0.0000001, +sqrt(1-x[i]^2)) $ value
  r
}

# Outer integral, over x from -1 to 1.

i2b <- function () 
  integrate(i1b,-1,1) $ value
